* This file is a part of nms_full_CALC
* It contains:
* subroutine nms_full_STU_check: compute STU parameters of the NMSSM
* double precision function nms_full_DZself: compute derivative of Z-selfenergy of the NMSSM
* double precision function nms_full_Yself: compute gamma-selfenergy  of the NMSSM
* double precision function nms_full_DYself: compute derivative of Z-selfenergy  of the NMSSM
* double precision function nms_full_Zself: compute Z-selfenergy  of the NMSSM
* double precision function nms_full_Wself: compute W-selfenergy  of the NMSSM
* double precision function nms_full_DWself: compute derivative of W-selfenergy  of the NMSSM
* subroutine nms_full_STU_UVcheck: check UV finite of STU parameters of the NMSSM
c last modified by ND 11/2022
c*********************************************************************c

      subroutine nms_full_STU_check(icheck)
      implicit none
      integer icheck
     
      double precision nms_full_Zself,nms_full_yself,nms_full_Wself,nms_full_yzself,
     &                nms_full_dzself,nms_full_dyself,nms_full_dwself,nms_full_dyzself,
     &                S,T,U
      double precision ismZself,ismyself,ismWself,ismyzself,
     &                ismdzself,ismdyself,ismdwself,ismdyzself

      double precision selfz0,selfw0,selfzmz,selfwmw,nZself
      double precision para_S,para_T,para_U
      common/lowen_observables/para_S,para_T,para_U      
#include "SM.h"
       MH2=125D0**2

       selfz0=(nms_full_Zself(0D0)-ismZself(0D0))/mz**2 
       selfw0=(nms_full_wself(0D0)-ismwself(0D0))/mw**2
       
       selfzmz=(nms_full_Zself(mz**2)-ismZself(mz**2))/mz**2
       selfwmw=(nms_full_wself(mw**2)-ismwself(mw**2))/mw**2


! using definition of S,T,U in 10.72a,10.72b,10.72c in 10. Electroweak Model and Constraints on New Physics
! Revised March 2020 by J. Erler (IF-UNAM; U. of Mainz) and A. Freitas (Pittsburg U.) in PDG
! web https://pdg.lbl.gov/2020/reviews/contents_sports.html

       T= (selfw0-selfz0)/alfa
       S=((selfzmz-selfz0) -(cw**2-sw**2)/cw/sw*(nms_full_yZself(mz**2)+ismyZself(mz**2))/mz**2
     &     -(nms_full_yself(mz**2)-ismyself(mz**2))/mz**2 )*4*sw**2*cw**2 /alfa  
       U =( (selfwmw-selfw0) - cw/sw*(nms_full_yZself(mz**2)+ismyZself(mz**2))/mz**2-
     &      (nms_full_yself(mz**2)-ismyself(mz**2))/mz**2 )*4*sw**2/alfa-S
       para_S=S;para_T=T;para_U=U
c       print*, ",nms_full_Zself(0D0),ismZself(0D0)",nms_full_Zself(0D0),ismZself(0D0)
c       print*, "selfz0,selfw0",selfz0,selfw0, (selfw0-selfz0)
c       print*, "nms_full_yZself(mz**2)-ismyZself(mz**2)",nms_full_yZself(mz**2),ismyZself(mz**2)
c       print*, "alfa",alfa,mz,mw
c       print*, "S,T,U=[-0.11,0.09],[-0.09,0.15],[-0.11,0.13] ",S,T,U
c       stop
        icheck=0
        if (S .gt. 0.09 .or. S.lt.-0.11) icheck=3
        if (T .gt. 0.15 .or. T.lt.-0.09) icheck=3
        if (U .gt. 0.13 .or. U.lt.-0.11) icheck=3
      end
!######################################################################
      subroutine nms_full_STU_check_old()
      implicit none
           
      double precision nms_full_Zself,nms_full_yself,nms_full_Wself,nms_full_yzself,
     &                nms_full_dzself,nms_full_dyself,nms_full_dwself,nms_full_dyzself,
     &                S,T,U
      double precision selfz,selfy,selfw,selfyz,
     &                 dselfz,dselfy,dselfw,dselfyz,selfQQ,self3Q,
     &                 self33,self11,dselfQQ,dself3Q,
     &                 dself33,dself11 

      double precision para_S,para_T,para_U
      common/lowen_observables/para_S,para_T,para_U      
#include "SM.h"
       selfz=nms_full_Zself(0D0)
       selfw=nms_full_wself(0D0)
       selfy=nms_full_yself(0D0)
       selfyz=-nms_full_yZself(0D0)

       dselfz=(nms_full_Zself(MZ2)-nms_full_Zself(0D0))/MZ2
       dselfw=(nms_full_wself(MW2)-nms_full_wself(0D0))/MW2
       dselfy=nms_full_dyself(0D0)
       dselfyz=-nms_full_dyZself(0D0)

c       selfQQ= selfy/e**2
c       self3Q=(cw*selfyz*sw)/e**2 + (selfy*sw**2)/e**2
       self11=(selfw*sw**2)/e**2
       self33=(cw**2*selfz*sw**2)/e**2 + 
     -    (2*cw*selfyz*sw**3)/e**2 + (selfy*sw**4)/e**2

c       dselfQQ= dselfy/e**2
       dself3Q=(cw*dselfyz*sw)/e**2 + (dselfy*sw**2)/e**2
       dself11=(dselfw*sw**2)/e**2
       dself33=(cw**2*dselfz*sw**2)/e**2 + 
     -    (2*cw*dselfyz*sw**3)/e**2 + (dselfy*sw**4)/e**2


       S= 16*pi*(dself33 -dself3Q)
       T= 4*pi/sw**2/cw**2/MZ2*(self11 -self33)
       U = 16*pi*(dself11 -dself33)
       para_S=S;para_T=T;para_U=U
c#define details
#ifdef details
       print*, "=======================nms_full_==========================="
       print*, "Follow definitions in  M. E. Peskin and T. Takeuchi 1992, "
       print*, "T", (selfw/MW2-selfz/MZ2 ) /alfa

       print*, "S",4*sw**2*cw**2*4*pi/e**2 *(dselfz - (cw**2-sw**2)/cw/sw*dselfyz -dselfy)
       print*, "U", 4*sw**2*4*pi/e**2*(dselfw - cw**2*dselfz - sw**2*dselfy- 2*cw*sw*dselfyz)
       print*, "selfz,selfw,selfy,selfyz",selfz,selfw,selfy,selfyz
       print*, "dselfz,dselfw,dselfy,dselfyz",dselfz,dselfw,dselfy,dselfyz
       print*, "S,T,U= ",S,T,U
       stop
#endif
      end

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function nms_full_Zself(p2)
        implicit none
        integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
       double precision myA0
       double complex myB0i
       double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
       common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3  
#include "HdecayTCLs.inc"
      self=0D0
        self=self+(4*CWWZ**2*myA0(MW2)-CWWZ**2*(2*MW2+5*P2)*
     &  myB0i(bb0,P2,MW2,MW2)+3*CW2*g2**2*myB0i(bb00,P2,0D0,0d0)+6*CW*g1*
     &  g2*SW*myB0i(bb00,P2,0D0,0d0)+3*g1**2*SW2*myB0i(bb00,P2,0D0,0d0)-8*
     &  CWWZ**2*myB0i(bb00,P2,MW2,MW2)-(3*CW2*g2**2*P2*
     &  myB0i(bb1,P2,0D0,0d0))/2d0-3*CW*g1*g2*P2*SW*myB0i(bb1,P2,0D0,0d0)-
     &  (3*g1**2*P2*SW2*myB0i(bb1,P2,0D0,0d0))/2d0-2*CWWZ**2*P2*
     &  myB0i(bb1,P2,MW2,MW2))
           do I2G3 = 1,2
       self=self+(CHpHmZZ(I2G3,I2G3)*myA0(MassHpm2(I2G3))+2*
     &  CHmWpZ(I2G3)**2*myB0i(bb0,P2,MW2,MassHpm2(I2G3))-4*CHpHmZ**2*
     &  myB0i(bb00,P2,MassHpm2(I2G3),MassHpm2(I2G3)))
           enddo
           do I2G3 = 1,2
           do I2G4 = 1,2
       self=self-2*(CF12ZL(I2G3,I2G4)*(-(CF12ZR(I2G4,I2G3)*MCha(I2G3)*
     &  MCha(I2G4)*myB0i(bb0,P2,MCha2(I2G3),MCha2(I2G4)))+
     &  CF12ZL(I2G4,I2G3)*(myA0(MCha2(I2G4))+MCha2(I2G3)*
     &  myB0i(bb0,P2,MCha2(I2G3),MCha2(I2G4))-2*
     &  myB0i(bb00,P2,MCha2(I2G3),MCha2(I2G4))+P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G4))))+CF12ZR(I2G3,I2G4)*(-
     &  (CF12ZL(I2G4,I2G3)*MCha(I2G3)*MCha(I2G4)*
     &  myB0i(bb0,P2,MCha2(I2G3),MCha2(I2G4)))+CF12ZR(I2G4,I2G3)*
     &  (myA0(MCha2(I2G4))+MCha2(I2G3)*myB0i(bb0,P2,MCha2(I2G3)
     &  ,MCha2(I2G4))-2*myB0i(bb00,P2,MCha2(I2G3),MCha2(I2G4))+P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G4)))))
           enddo
           enddo
           do I3G3 = 1,3
       self=self+(-6*(CddZL**2+CddZR**2)*myA0(MFd2(I3G3))-2*(ctautauZL*
     &  *2+ctautauZR**2)*myA0(MFe2(I3G3))-6*(CuuZL**2+CuuZR**2)*
     &  myA0(MFu2(I3G3))+CSvSvZZ*myA0(MSv2(I3G3))-6*(CddZL-CddZR)**2*
     &  MFd2(I3G3)*myB0i(bb0,P2,MFd2(I3G3),MFd2(I3G3))-2*(ctautauZL-
     &  ctautauZR)**2*MFe2(I3G3)*myB0i(bb0,P2,MFe2(I3G3),MFe2(I3G3))-6*
     &  (CuuZL-CuuZR)**2*MFu2(I3G3)*myB0i(bb0,P2,MFu2(I3G3),MFu2(I3G3))
     &  +12*(CddZL**2+CddZR**2)*myB0i(bb00,P2,MFd2(I3G3),MFd2(I3G3))+4*
     &  (ctautauZL**2+ctautauZR**2)*myB0i(bb00,P2,MFe2(I3G3),MFe2(I3G3)
     &  )+12*(CuuZL**2+CuuZR**2)*myB0i(bb00,P2,MFu2(I3G3),MFu2(I3G3))-
     &  4*CsvsvZ**2*myB0i(bb00,P2,MSv2(I3G3),MSv2(I3G3))-6*(CddZL**2+
     &  CddZR**2)*P2*myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))-2*(ctautauZL**
     &  2+ctautauZR**2)*P2*myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))-6*
     &  (CuuZL**2+CuuZR**2)*P2*myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3)))
           enddo
           do I5G3 = 1,5
           do I5G4 = 1,5
       self=self+(-(CF11ZL(I5G3,I5G4)*(-(CF11ZR(I5G4,I5G3)*MNeu(I5G3)*
     &  MNeu(I5G4)*myB0i(bb0,P2,MNeu2(I5G3),MNeu2(I5G4)))+
     &  CF11ZL(I5G4,I5G3)*(myA0(MNeu2(I5G4))+MNeu2(I5G3)*
     &  myB0i(bb0,P2,MNeu2(I5G3),MNeu2(I5G4))-2*
     &  myB0i(bb00,P2,MNeu2(I5G3),MNeu2(I5G4))+P2*
     &  myB0i(bb1,P2,MNeu2(I5G3),MNeu2(I5G4)))))-CF11ZR(I5G3,I5G4)*(-
     &  (CF11ZL(I5G4,I5G3)*MNeu(I5G3)*MNeu(I5G4)*
     &  myB0i(bb0,P2,MNeu2(I5G3),MNeu2(I5G4)))+CF11ZR(I5G4,I5G3)*
     &  (myA0(MNeu2(I5G4))+MNeu2(I5G3)*myB0i(bb0,P2,MNeu2(I5G3)
     &  ,MNeu2(I5G4))-2*myB0i(bb00,P2,MNeu2(I5G3),MNeu2(I5G4))+P2*
     &  myB0i(bb1,P2,MNeu2(I5G3),MNeu2(I5G4)))))
           enddo
           enddo
           do I6G3 = 1,6
       self=self+(-(IC*ChhZZ(I6G3,I6G3)*myA0(Masshh2(I6G3)))/2d0+3*
     &  CSdSdZZ(I6G3,I6G3)*myA0(MSd2(I6G3))+CSeSeZZ(I6G3,I6G3)*
     &  myA0(MSe2(I6G3))+3*CSuSuZZ(I6G3,I6G3)*myA0(MSu2(I6G3))-
     &  ChZZ(I6G3)**2*myB0i(bb0,P2,MZ2,Masshh2(I6G3)))
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self-2*(ChhZ(I6G3,I6G4)**2*myB0i(bb00,P2,Masshh2(I6G3)
     &  ,Masshh2(I6G4))+6*CsdmsdpZ(I6G3,I6G4)*CsdmsdpZ(I6G4,I6G3)*
     &  myB0i(bb00,P2,MSd2(I6G3),MSd2(I6G4))+2*CsemsepZ(I6G3,I6G4)*
     &  CsemsepZ(I6G4,I6G3)*myB0i(bb00,P2,MSe2(I6G3),MSe2(I6G4))+6*
     &  CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3)*
     &  myB0i(bb00,P2,MSu2(I6G3),MSu2(I6G4)))
           enddo
           enddo


       nms_full_Zself=self*fac
      end

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function nms_full_DZself(p2)
        implicit none
       integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
       double precision myA0
       double complex myB0i
       double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
       common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3 
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00

#include "HdecayTCLs.inc"
      Dself=0D0
        Dself=Dself+(-10*CWWZ**2*myB0i(bb0,P2,MW2,MW2)-3*(CW2*g2**2+2*
     &  CW*g1*g2*SW+g1**2*SW2)*myB0i(bb1,P2,0D0,0d0)-4*CWWZ**2*
     &  myB0i(bb1,P2,MW2,MW2)-4*CWWZ**2*MW2*myB0i(dbb0,P2,MW2,MW2)-10*
     &  CWWZ**2*P2*myB0i(dbb0,P2,MW2,MW2)+6*CW2*g2**2*
     &  myB0i(dbb00,P2,0D0,0d0)+12*CW*g1*g2*SW*myB0i(dbb00,P2,0D0,0d0)+6*
     &  g1**2*SW2*myB0i(dbb00,P2,0D0,0d0)-16*CWWZ**2*
     &  myB0i(dbb00,P2,MW2,MW2)-3*CW2*g2**2*P2*myB0i(dbb1,P2,0D0,0d0)-6*
     &  CW*g1*g2*P2*SW*myB0i(dbb1,P2,0D0,0d0)-3*g1**2*P2*SW2*
     &  myB0i(dbb1,P2,0D0,0d0)-4*CWWZ**2*P2*myB0i(dbb1,P2,MW2,MW2))/2d0
           do I2G3 = 1,2
       Dself=Dself+(2*CHmWpZ(I2G3)**2*myB0i(dbb0,P2,MW2,MassHpm2(I2G3))
     &  -4*CHpHmZ**2*myB0i(dbb00,P2,MassHpm2(I2G3),MassHpm2(I2G3)))
           enddo
           do I2G3 = 1,2
           do I2G4 = 1,2
       Dself=Dself-2*(CF12ZL(I2G3,I2G4)*(-(CF12ZR(I2G4,I2G3)*
     &  MCha(I2G3)*MCha(I2G4)*myB0i(dbb0,P2,MCha2(I2G3),MCha2(I2G4)))+
     &  CF12ZL(I2G4,I2G3)*(myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G4))+
     &  MCha2(I2G3)*myB0i(dbb0,P2,MCha2(I2G3),MCha2(I2G4))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G4))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G4))))+CF12ZR(I2G3,I2G4)*(-
     &  (CF12ZL(I2G4,I2G3)*MCha(I2G3)*MCha(I2G4)*
     &  myB0i(dbb0,P2,MCha2(I2G3),MCha2(I2G4)))+CF12ZR(I2G4,I2G3)*
     &  (myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G4))+MCha2(I2G3)*
     &  myB0i(dbb0,P2,MCha2(I2G3),MCha2(I2G4))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G4))+P2*myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G4)))))
           enddo
           enddo
           do I3G3 = 1,3
       Dself=Dself-2*(3*(CddZL**2+CddZR**2)*myB0i(bb1,P2,MFd2(I3G3)
     &  ,MFd2(I3G3))+(ctautauZL**2+ctautauZR**2)*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))+3*(CuuZL**2+CuuZR**2)*
     &  myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3))+3*(CddZL-CddZR)**2*
     &  MFd2(I3G3)*myB0i(dbb0,P2,MFd2(I3G3),MFd2(I3G3))+(ctautauZL-
     &  ctautauZR)**2*MFe2(I3G3)*myB0i(dbb0,P2,MFe2(I3G3),MFe2(I3G3))+
     &  3*(CuuZL-CuuZR)**2*MFu2(I3G3)*myB0i(dbb0,P2,MFu2(I3G3)
     &  ,MFu2(I3G3))-6*(CddZL**2+CddZR**2)*myB0i(dbb00,P2,MFd2(I3G3)
     &  ,MFd2(I3G3))-2*(ctautauZL**2+ctautauZR**2)*
     &  myB0i(dbb00,P2,MFe2(I3G3),MFe2(I3G3))-6*(CuuZL**2+CuuZR**2)*
     &  myB0i(dbb00,P2,MFu2(I3G3),MFu2(I3G3))+2*CsvsvZ**2*
     &  myB0i(dbb00,P2,MSv2(I3G3),MSv2(I3G3))+3*(CddZL**2+CddZR**2)*P2*
     &  myB0i(dbb1,P2,MFd2(I3G3),MFd2(I3G3))+(ctautauZL**2+ctautauZR**
     &  2)*P2*myB0i(dbb1,P2,MFe2(I3G3),MFe2(I3G3))+3*(CuuZL**2+CuuZR**
     &  2)*P2*myB0i(dbb1,P2,MFu2(I3G3),MFu2(I3G3)))
           enddo
           do I5G3 = 1,5
           do I5G4 = 1,5
       Dself=Dself+(-(CF11ZL(I5G3,I5G4)*(-(CF11ZR(I5G4,I5G3)*MNeu(I5G3)
     &  *MNeu(I5G4)*myB0i(dbb0,P2,MNeu2(I5G3),MNeu2(I5G4)))+
     &  CF11ZL(I5G4,I5G3)*(myB0i(bb1,P2,MNeu2(I5G3),MNeu2(I5G4))+
     &  MNeu2(I5G3)*myB0i(dbb0,P2,MNeu2(I5G3),MNeu2(I5G4))-2*
     &  myB0i(dbb00,P2,MNeu2(I5G3),MNeu2(I5G4))+P2*
     &  myB0i(dbb1,P2,MNeu2(I5G3),MNeu2(I5G4)))))-CF11ZR(I5G3,I5G4)*(-
     &  (CF11ZL(I5G4,I5G3)*MNeu(I5G3)*MNeu(I5G4)*
     &  myB0i(dbb0,P2,MNeu2(I5G3),MNeu2(I5G4)))+CF11ZR(I5G4,I5G3)*
     &  (myB0i(bb1,P2,MNeu2(I5G3),MNeu2(I5G4))+MNeu2(I5G3)*
     &  myB0i(dbb0,P2,MNeu2(I5G3),MNeu2(I5G4))-2*
     &  myB0i(dbb00,P2,MNeu2(I5G3),MNeu2(I5G4))+P2*myB0i(dbb1,P2,MNeu2(I5G3),MNeu2(I5G4)))))
           enddo
           enddo
           do I6G3 = 1,6
       Dself=Dself-(ChZZ(I6G3)**2*myB0i(dbb0,P2,MZ2,Masshh2(I6G3)))
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself-2*(ChhZ(I6G3,I6G4)**2*myB0i(dbb00,P2,Masshh2(I6G3)
     &  ,Masshh2(I6G4))+6*CsdmsdpZ(I6G3,I6G4)*CsdmsdpZ(I6G4,I6G3)*
     &  myB0i(dbb00,P2,MSd2(I6G3),MSd2(I6G4))+2*CsemsepZ(I6G3,I6G4)*
     &  CsemsepZ(I6G4,I6G3)*myB0i(dbb00,P2,MSe2(I6G3),MSe2(I6G4))+6*
     &  CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3)*
     &  myB0i(dbb00,P2,MSu2(I6G3),MSu2(I6G4)))
           enddo
           enddo
      nms_full_DZself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function nms_full_Yself(p2)
        implicit none
       integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
       double precision myA0
       double complex myB0i
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00

#include "HdecayTCLs.inc"
      self=0D0
        self=self-2*((4*alpha*Pi-3*g2**2*SW2)*myA0(MW2)+2*alpha*Pi*((2*
     &  MW2+5*P2)*myB0i(bb0,P2,MW2,MW2)+8*myB0i(bb00,P2,MW2,MW2)+2*P2*
     &  myB0i(bb1,P2,MW2,MW2)))
           do I2G3 = 1,2
       self=self+(4*CHmWpA(I2G3)**2*myB0i(bb0,P2,MW2,MassHpm2(I2G3))-
     &  32*alpha*Pi*(myA0(MCha2(I2G3))+myB0i(bb00,P2,MassHpm2(I2G3)
     &  ,MassHpm2(I2G3)))+64*alpha*Pi*myB0i(bb00,P2,MCha2(I2G3)
     &  ,MCha2(I2G3))-32*alpha*P2*Pi*myB0i(bb1,P2,MCha2(I2G3)
     &  ,MCha2(I2G3))+(CW2*g1**2+g2*(2*CW*g1*SW+g2*SW2))*
     &  myA0(MassHpm2(I2G3))*(ZP(I2G3,1)**2+ZP(I2G3,2)**2))/2d0
           enddo
           do I3G3 = 1,3
       self=self+(-16*alpha*Pi*(myA0(MFd2(I3G3))+3*myA0(MFe2(I3G3))+4*
     &  myA0(MFu2(I3G3))-2*myB0i(bb00,P2,MFd2(I3G3),MFd2(I3G3))-6*
     &  myB0i(bb00,P2,MFe2(I3G3),MFe2(I3G3))-8*myB0i(bb00,P2,MFu2(I3G3)
     &  ,MFu2(I3G3))+P2*myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+3*P2*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))+4*P2*
     &  myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3))))/3d0
           enddo
           do I6G3 = 1,6
       self=self+(3*CSdmSdpAA(I6G3,I6G3)*myA0(MSd2(I6G3))+
     &  CSemSepAA(I6G3,I6G3)*myA0(MSe2(I6G3))+3*CSumSupAA(I6G3,I6G3)*
     &  myA0(MSu2(I6G3))-(16*alpha*Pi*myB0i(bb00,P2,MSd2(I6G3)
     &  ,MSd2(I6G3)))/3d0-16*alpha*Pi*myB0i(bb00,P2,MSe2(I6G3)
     &  ,MSe2(I6G3))-(64*alpha*Pi*myB0i(bb00,P2,MSu2(I6G3),MSu2(I6G3)))
     &  /3d0)
           enddo
      nms_full_Yself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function nms_full_DYself(p2)
        implicit none
       integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
       double precision myA0
       double complex myB0i
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00

#include "HdecayTCLs.inc"
      Dself=0D0
        Dself=Dself-4*alpha*Pi*(5*myB0i(bb0,P2,MW2,MW2)+2*
     &  myB0i(bb1,P2,MW2,MW2)+2*MW2*myB0i(dbb0,P2,MW2,MW2)+5*P2*
     &  myB0i(dbb0,P2,MW2,MW2)+8*myB0i(dbb00,P2,MW2,MW2)+2*P2*
     &  myB0i(dbb1,P2,MW2,MW2))
           do I2G3 = 1,2
       Dself=Dself-2*(8*alpha*Pi*myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3))-
     &  CHmWpA(I2G3)**2*myB0i(dbb0,P2,MW2,MassHpm2(I2G3))+8*alpha*Pi*
     &  (myB0i(dbb00,P2,MassHpm2(I2G3),MassHpm2(I2G3))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G3))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G3))))
           enddo
           do I3G3 = 1,3
       Dself=Dself+(-16*alpha*Pi*(myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+3*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))+4*myB0i(bb1,P2,MFu2(I3G3)
     &  ,MFu2(I3G3))-2*myB0i(dbb00,P2,MFd2(I3G3),MFd2(I3G3))-6*
     &  myB0i(dbb00,P2,MFe2(I3G3),MFe2(I3G3))-8*
     &  myB0i(dbb00,P2,MFu2(I3G3),MFu2(I3G3))+P2*
     &  myB0i(dbb1,P2,MFd2(I3G3),MFd2(I3G3))+3*P2*
     &  myB0i(dbb1,P2,MFe2(I3G3),MFe2(I3G3))+4*P2*
     &  myB0i(dbb1,P2,MFu2(I3G3),MFu2(I3G3))))/3d0
           enddo
           do I6G3 = 1,6
       Dself=Dself+(-16*alpha*Pi*(myB0i(dbb00,P2,MSd2(I6G3),MSd2(I6G3))+
     &  3*myB0i(dbb00,P2,MSe2(I6G3),MSe2(I6G3))+4*
     &  myB0i(dbb00,P2,MSu2(I6G3),MSu2(I6G3))))/3d0
           enddo
      nms_full_DYself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function nms_full_YZself(p2)
      implicit none
       integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
       double precision myA0
       double complex myB0i
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision ZH5(5,5),ZH(6,6)
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00

#include "HdecayTCLs.inc"
      self=0D0
        self=self+((-2*CWWZ*e+3*g2**2*S2W)*myA0(MW2)-CWWZ*e*((2*MW2+5*
     &  P2)*myB0i(bb0,P2,MW2,MW2)+8*myB0i(bb00,P2,MW2,MW2)+2*P2*
     &  myB0i(bb1,P2,MW2,MW2)))
           do I2G3 = 1,2
       self=self+(-8*e*(CF12ZL(I2G3,I2G3)+CF12ZR(I2G3,I2G3))*
     &  myA0(MCha2(I2G3))-8*CHmWpA(I2G3)*CHmWpZ(I2G3)*
     &  myB0i(bb0,P2,MW2,MassHpm2(I2G3))-16*CHpHmZ*e*
     &  myB0i(bb00,P2,MassHpm2(I2G3),MassHpm2(I2G3))+16*e*
     &  (CF12ZL(I2G3,I2G3)+CF12ZR(I2G3,I2G3))*myB0i(bb00,P2,MCha2(I2G3)
     &  ,MCha2(I2G3))-8*e*P2*(CF12ZL(I2G3,I2G3)+CF12ZR(I2G3,I2G3))*
     &  myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3))+(2*C2W*g1*g2+(-g1**2+g2**
     &  2)*S2W)*myA0(MassHpm2(I2G3))*(ZP(I2G3,1)**2+ZP(I2G3,2)**2))/4d0
           enddo
           do I3G3 = 1,3
       self=self-2*e*((CddZL+CddZR)*myA0(MFd2(I3G3))+(ctautauZL+
     &  ctautauZR)*myA0(MFe2(I3G3))-2*CuuZL*myA0(MFu2(I3G3))-2*CuuZR*
     &  myA0(MFu2(I3G3))-2*CddZL*myB0i(bb00,P2,MFd2(I3G3),MFd2(I3G3))-
     &  2*CddZR*myB0i(bb00,P2,MFd2(I3G3),MFd2(I3G3))-2*ctautauZL*
     &  myB0i(bb00,P2,MFe2(I3G3),MFe2(I3G3))-2*ctautauZR*
     &  myB0i(bb00,P2,MFe2(I3G3),MFe2(I3G3))+4*CuuZL*
     &  myB0i(bb00,P2,MFu2(I3G3),MFu2(I3G3))+4*CuuZR*
     &  myB0i(bb00,P2,MFu2(I3G3),MFu2(I3G3))+CddZL*P2*
     &  myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+CddZR*P2*
     &  myB0i(bb1,P2,MFd2(I3G3),MFd2(I3G3))+ctautauZL*P2*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))+ctautauZR*P2*
     &  myB0i(bb1,P2,MFe2(I3G3),MFe2(I3G3))-2*CuuZL*P2*
     &  myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3))-2*CuuZR*P2*
     &  myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3)))
           enddo
           do I6G3 = 1,6
       self=self+(3*CSdmSdpAZ(I6G3,I6G3)*myA0(MSd2(I6G3))+
     &  CSemSepAZ(I6G3,I6G3)*myA0(MSe2(I6G3))+3*CSumSupAZ(I6G3,I6G3)*
     &  myA0(MSu2(I6G3))-4*e*CsdmsdpZ(I6G3,I6G3)*
     &  myB0i(bb00,P2,MSd2(I6G3),MSd2(I6G3))-4*e*CsemsepZ(I6G3,I6G3)*
     &  myB0i(bb00,P2,MSe2(I6G3),MSe2(I6G3))+8*e*CsupsumZ(I6G3,I6G3)*
     &  myB0i(bb00,P2,MSu2(I6G3),MSu2(I6G3)))
           enddo

      nms_full_YZself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function nms_full_DYZself(p2)
      implicit none
       integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
       double precision myA0
       double complex myB0i
       double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
       common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3       
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself

      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00

#include "HdecayTCLs.inc"
      Dself=0D0
        Dself=Dself-(CWWZ*e*(5*myB0i(bb0,P2,MW2,MW2)+2*
     &  myB0i(bb1,P2,MW2,MW2)+2*MW2*myB0i(dbb0,P2,MW2,MW2)+5*P2*
     &  myB0i(dbb0,P2,MW2,MW2)+8*myB0i(dbb00,P2,MW2,MW2)+2*P2*
     &  myB0i(dbb1,P2,MW2,MW2)))
           do I2G3 = 1,2
       Dself=Dself-2*(CHmWpA(I2G3)*CHmWpZ(I2G3)*
     &  myB0i(dbb0,P2,MW2,MassHpm2(I2G3))+2*CHpHmZ*e*
     &  myB0i(dbb00,P2,MassHpm2(I2G3),MassHpm2(I2G3))+e*
     &  CF12ZL(I2G3,I2G3)*(myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G3))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G3)))+e*CF12ZR(I2G3,I2G3)*
     &  (myB0i(bb1,P2,MCha2(I2G3),MCha2(I2G3))-2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MCha2(I2G3))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MCha2(I2G3))))
           enddo
           do I3G3 = 1,3
       Dself=Dself+2*e*(-((CddZL+CddZR)*myB0i(bb1,P2,MFd2(I3G3)
     &  ,MFd2(I3G3)))-(ctautauZL+ctautauZR)*myB0i(bb1,P2,MFe2(I3G3)
     &  ,MFe2(I3G3))+2*(CuuZL+CuuZR)*myB0i(bb1,P2,MFu2(I3G3),MFu2(I3G3)
     &  )+2*(CddZL+CddZR)*myB0i(dbb00,P2,MFd2(I3G3),MFd2(I3G3))+2*
     &  (ctautauZL+ctautauZR)*myB0i(dbb00,P2,MFe2(I3G3),MFe2(I3G3))-4*
     &  (CuuZL+CuuZR)*myB0i(dbb00,P2,MFu2(I3G3),MFu2(I3G3))-(CddZL+
     &  CddZR)*P2*myB0i(dbb1,P2,MFd2(I3G3),MFd2(I3G3))-(ctautauZL+
     &  ctautauZR)*P2*myB0i(dbb1,P2,MFe2(I3G3),MFe2(I3G3))+2*(CuuZL+
     &  CuuZR)*P2*myB0i(dbb1,P2,MFu2(I3G3),MFu2(I3G3)))
           enddo
           do I6G3 = 1,6
       Dself=Dself-4*e*(CsdmsdpZ(I6G3,I6G3)*myB0i(dbb00,P2,MSd2(I6G3)
     &  ,MSd2(I6G3))+CsemsepZ(I6G3,I6G3)*myB0i(dbb00,P2,MSe2(I6G3)
     &  ,MSe2(I6G3))-2*CsupsumZ(I6G3,I6G3)*myB0i(dbb00,P2,MSu2(I6G3)
     &  ,MSu2(I6G3)))
           enddo

      nms_full_DYZself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function nms_full_Wself(p2)
      implicit none
       integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
       double precision myA0
       double complex myB0i
       double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00

#include "HdecayTCLs.inc"
       self=0D0
        self=self+((-2*CWWZ**2+3*g2**2-8*alpha*Pi)*myA0(MW2)+3*CWWZ**2*
     &  myA0(MZ2)-20*alpha*P2*Pi*myB0i(bb0,P2,0D0,MW2)-2*CWWZ**2*MZ2*
     &  myB0i(bb0,P2,MW2,MZ2)-3*CWWZ**2*P2*myB0i(bb0,P2,MW2,MZ2)-32*
     &  alpha*Pi*myB0i(bb00,P2,0D0,MW2)-8*CWWZ**2*myB0i(bb00,P2,MW2,MZ2)
     &  -8*alpha*P2*Pi*myB0i(bb1,P2,0D0,MW2)+2*CWWZ**2*P2*
     &  myB0i(bb1,P2,MW2,MZ2))
           do I2G3 = 1,2
       self=self+(CHmHpWW(I2G3,I2G3)*myA0(MassHpm2(I2G3))+CHmWpA(I2G3)*
     &  *2*myB0i(bb0,P2,0D0,MassHpm2(I2G3))+CHmWpZ(I2G3)**2*
     &  myB0i(bb0,P2,MZ2,MassHpm2(I2G3)))
           enddo
           do I3G3 = 1,3
       self=self+(g2**2*(-2*myA0(MFe2(I3G3))+myA0(MSv2(I3G3))+4*
     &  myB0i(bb00,P2,0D0,MFe2(I3G3))-2*P2*myB0i(bb1,P2,0D0,MFe2(I3G3))
     &  ))/2d0
           enddo
           do I3G3 = 1,3
           do I3G4 = 1,3
       self=self+6*CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)*(-myA0(MFd2(I3G4))
     &  +(P2-MFu2(I3G3))*myB0i(bb0,P2,MFd2(I3G4),MFu2(I3G3))+2*
     &  myB0i(bb00,P2,MFd2(I3G4),MFu2(I3G3))+P2*myB0i(bb1,P2,MFd2(I3G4)
     &  ,MFu2(I3G3)))
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       self=self+2*(CLF12F11W(I2G3,I5G3)*(DCONJG(CRF12F11W(I2G3,I5G3))*
     &  MCha(I2G3)*MNeu(I5G3)*myB0i(bb0,P2,MCha2(I2G3),MNeu2(I5G3))+
     &  DCONJG(CLF12F11W(I2G3,I5G3))*(-myA0(MCha2(I2G3))+(P2-
     &  MNeu2(I5G3))*myB0i(bb0,P2,MCha2(I2G3),MNeu2(I5G3))+2*
     &  myB0i(bb00,P2,MCha2(I2G3),MNeu2(I5G3))+P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MNeu2(I5G3))))+CRF12F11W(I2G3,I5G3)*
     &  (DCONJG(CLF12F11W(I2G3,I5G3))*MCha(I2G3)*MNeu(I5G3)*
     &  myB0i(bb0,P2,MCha2(I2G3),MNeu2(I5G3))+
     &  DCONJG(CRF12F11W(I2G3,I5G3))*(-myA0(MCha2(I2G3))+(P2-
     &  MNeu2(I5G3))*myB0i(bb0,P2,MCha2(I2G3),MNeu2(I5G3))+2*
     &  myB0i(bb00,P2,MCha2(I2G3),MNeu2(I5G3))+P2*
     &  myB0i(bb1,P2,MCha2(I2G3),MNeu2(I5G3)))))
           enddo
           enddo
           do I6G3 = 1,6
       self=self+(-(IC*ChhWmWp(I6G3,I6G3)*myA0(Masshh2(I6G3)))/2d0+3*
     &  CSdSdWW(I6G3,I6G3)*myA0(MSd2(I6G3))+CSeSeWW(I6G3,I6G3)*
     &  myA0(MSe2(I6G3))+3*CSuSuWW(I6G3,I6G3)*myA0(MSu2(I6G3))-
     &  ChWmWp(I6G3)**2*myB0i(bb0,P2,MW2,Masshh2(I6G3)))
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       self=self+4*ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3)*
     &  myB0i(bb00,P2,Masshh2(I6G3),MassHpm2(I2G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I6G3 = 1,6
       self=self-4*CsesvW(I6G3,I3G3)*DCONJG(CsesvW(I6G3,I3G3))*
     &  myB0i(bb00,P2,MSe2(I6G3),MSv2(I3G3))
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self-12*CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))*
     &  myB0i(bb00,P2,MSd2(I6G4),MSu2(I6G3))
           enddo
           enddo
      nms_full_Wself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function nms_full_DWself(p2)
      implicit none
       integer I2G3,I2G4,I3G3,I3G4,I5G3,I5G4,I6G3,I6G4
       double precision myA0
       double complex myB0i      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00

#include "HdecayTCLs.inc"
      Dself=0D0
        Dself=Dself+(-20*alpha*Pi*myB0i(bb0,P2,0D0,MW2)-3*CWWZ**2*
     &  myB0i(bb0,P2,MW2,MZ2)-8*alpha*Pi*myB0i(bb1,P2,0D0,MW2)+2*CWWZ**
     &  2*myB0i(bb1,P2,MW2,MZ2)-20*alpha*P2*Pi*myB0i(dbb0,P2,0D0,MW2)-2*
     &  CWWZ**2*MZ2*myB0i(dbb0,P2,MW2,MZ2)-3*CWWZ**2*P2*
     &  myB0i(dbb0,P2,MW2,MZ2)-32*alpha*Pi*myB0i(dbb00,P2,0D0,MW2)-8*
     &  CWWZ**2*myB0i(dbb00,P2,MW2,MZ2)-8*alpha*P2*Pi*
     &  myB0i(dbb1,P2,0D0,MW2)+2*CWWZ**2*P2*myB0i(dbb1,P2,MW2,MZ2))
           do I2G3 = 1,2
       Dself=Dself+(CHmWpA(I2G3)**2*myB0i(dbb0,P2,0D0,MassHpm2(I2G3))+
     &  CHmWpZ(I2G3)**2*myB0i(dbb0,P2,MZ2,MassHpm2(I2G3)))
           enddo
           do I3G3 = 1,3
       Dself=Dself-(g2**2*(myB0i(bb1,P2,0D0,MFe2(I3G3))-2*
     &  myB0i(dbb00,P2,0D0,MFe2(I3G3))+P2*myB0i(dbb1,P2,0D0,MFe2(I3G3))
     &  ))
           enddo
           do I3G3 = 1,3
           do I3G4 = 1,3
       Dself=Dself+6*CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)*
     &  (myB0i(bb0,P2,MFd2(I3G4),MFu2(I3G3))+myB0i(bb1,P2,MFd2(I3G4)
     &  ,MFu2(I3G3))+P2*myB0i(dbb0,P2,MFd2(I3G4),MFu2(I3G3))-MFu2(I3G3)
     &  *myB0i(dbb0,P2,MFd2(I3G4),MFu2(I3G3))+2*
     &  myB0i(dbb00,P2,MFd2(I3G4),MFu2(I3G3))+P2*
     &  myB0i(dbb1,P2,MFd2(I3G4),MFu2(I3G3)))
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       Dself=Dself+2*(CLF12F11W(I2G3,I5G3)*(DCONJG(CRF12F11W(I2G3,I5G3)
     &  )*MCha(I2G3)*MNeu(I5G3)*myB0i(dbb0,P2,MCha2(I2G3),MNeu2(I5G3))+
     &  DCONJG(CLF12F11W(I2G3,I5G3))*(myB0i(bb0,P2,MCha2(I2G3)
     &  ,MNeu2(I5G3))+myB0i(bb1,P2,MCha2(I2G3),MNeu2(I5G3))+P2*
     &  myB0i(dbb0,P2,MCha2(I2G3),MNeu2(I5G3))-MNeu2(I5G3)*
     &  myB0i(dbb0,P2,MCha2(I2G3),MNeu2(I5G3))+2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MNeu2(I5G3))+P2*
     &  myB0i(dbb1,P2,MCha2(I2G3),MNeu2(I5G3))))+CRF12F11W(I2G3,I5G3)*
     &  (DCONJG(CLF12F11W(I2G3,I5G3))*MCha(I2G3)*MNeu(I5G3)*
     &  myB0i(dbb0,P2,MCha2(I2G3),MNeu2(I5G3))+
     &  DCONJG(CRF12F11W(I2G3,I5G3))*(myB0i(bb0,P2,MCha2(I2G3)
     &  ,MNeu2(I5G3))+myB0i(bb1,P2,MCha2(I2G3),MNeu2(I5G3))+P2*
     &  myB0i(dbb0,P2,MCha2(I2G3),MNeu2(I5G3))-MNeu2(I5G3)*
     &  myB0i(dbb0,P2,MCha2(I2G3),MNeu2(I5G3))+2*
     &  myB0i(dbb00,P2,MCha2(I2G3),MNeu2(I5G3))+P2*myB0i(dbb1,P2,MCha2(I2G3),MNeu2(I5G3)))))
           enddo
           enddo
           do I6G3 = 1,6
       Dself=Dself-(ChWmWp(I6G3)**2*myB0i(dbb0,P2,MW2,Masshh2(I6G3)))
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       Dself=Dself+4*ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3)*
     &  myB0i(dbb00,P2,Masshh2(I6G3),MassHpm2(I2G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I6G3 = 1,6
       Dself=Dself-4*CsesvW(I6G3,I3G3)*DCONJG(CsesvW(I6G3,I3G3))*
     &  myB0i(dbb00,P2,MSe2(I6G3),MSv2(I3G3))
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself-12*CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))*
     &  myB0i(dbb00,P2,MSd2(I6G4),MSu2(I6G3))
           enddo
           enddo
      nms_full_DWself=Dself*fac
      end

